//
// Software License for AMDiS
//
// Copyright (c) 2010 Dresden University of Technology 
// All rights reserved.
// Authors: Simon Vey, Thomas Witkowski et al.
//
// This file is part of AMDiS
//
// See also license.opensource.txt in the distribution.


#include "DofWriter.h"
#include "DOFVector.h"
#include "BasisFunction.h"

namespace AMDiS {

  using namespace std;

  void DofWriter::writeFile(string filename, vector<DOFVector<double>*> &vec)
  {
    FUNCNAME("DofWriter::writeFile()");

    DOFVector<WorldVector<double> > coordDof(feSpace, "tmp");
    
    Mesh *mesh = feSpace->getMesh();
    const BasisFunction* basFcts = feSpace->getBasisFcts();
    int nBasFcts = basFcts->getNumber();
    vector<DegreeOfFreedom> dofVec(nBasFcts);
    
    TraverseStack stack;
    ElInfo *elInfo = 
      stack.traverseFirst(mesh, -1, Mesh::CALL_LEAF_EL | Mesh::FILL_COORDS);
    while (elInfo) {
      basFcts->getLocalIndices(elInfo->getElement(), 
			       feSpace->getAdmin(), dofVec);
      for (int i = 0; i < nBasFcts; i++) {
	DimVec<double> *baryCoords = basFcts->getCoords(i);
	elInfo->coordToWorld(*baryCoords, coordDof[dofVec[i]]);
      }
      
      elInfo = stack.traverseNext(elInfo);
    }   
    
    ofstream outfile;
    outfile.open(filename.c_str());
    outfile.precision(10);
    
    DOFIterator<WorldVector<double> > it(&coordDof, USED_DOFS);
    for (it.reset(); !it.end(); ++it) {
      outfile << it.getDOFIndex() << " ";
      
      for (int i = 0; i < Global::getGeo(WORLD); i++)
	outfile << (*it)[i] << " ";
      
      for (unsigned int i = 0; i < vec.size(); i++) 
	outfile << (*(vec[i]))[it.getDOFIndex()] << " ";
      outfile << endl;
    }   
    
    outfile.close();
  }
    
}
